﻿using System.Reflection;
using Bunny.Common.Attribute;
using Bunny.Common.Context.Middleware;
using Bunny.Dao.Common.Constant;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Controllers;
using Microsoft.AspNetCore.Mvc.Filters;
using Newtonsoft.Json;
using Newtonsoft.Json.Serialization;
using IDatabase = StackExchange.Redis.IDatabase;

namespace Bunny.Common.Filter;

/// <summary>
///     Redis缓存注解
/// </summary>
public class CacheIiAsyncActionFilter : IAsyncActionFilter
{
    private readonly IDatabase _redisDatabase = RedisContext.RedisDatabase!;

    public async Task OnActionExecutionAsync(ActionExecutingContext context, ActionExecutionDelegate next)
    {
        var actionDesc = context.ActionDescriptor as ControllerActionDescriptor;
        var hasCacheableAttribute = actionDesc?.MethodInfo.IsDefined(typeof(CacheableAttribute), false) ?? false;

        // 如果没有加注解直接返回
        if (!hasCacheableAttribute)
        {
            await next();
            return;
        }

        // 获取注解中的值
        var cacheableAttribute = actionDesc?.MethodInfo.GetCustomAttribute<CacheableAttribute>();

        // 注解为空
        if (cacheableAttribute?.Key == null)
        {
            await next();
            return;
        }

        // 缓存的key
        var cacheKey = cacheableAttribute.Key;
        // 缓存时间
        var duration = cacheableAttribute.TimeDuration;

        // 缓存数据
        var cachedData = _redisDatabase.StringGet(cacheKey);

        // 如果注解中有值将值赋值给返回结果
        if (!cachedData.IsNullOrEmpty)
        {
            context.Result = new ContentResult
            {
                Content = cachedData.ToString(),
                ContentType = "application/json"
            };
            return;
        }

        // 如果注解中没有值，赋值内容
        var result = (ObjectResult)next().GetAwaiter().GetResult().Result!;
        var jsonSettings = new JsonSerializerSettings
        {
            DateFormatString = LocalDateTimeConstant.DefaultDateTimeSecondFormat,
            ContractResolver = new CamelCasePropertyNamesContractResolver()
        };
        // 现在你可以访问 result.Data
        var json = JsonConvert.SerializeObject(result.Value, jsonSettings);
        _redisDatabase.StringSet(cacheKey, json, duration);
    }
}