﻿using System.Transactions;
using Bunny.Common.Attribute;
using Bunny.Common.Context;
using log4net;
using Microsoft.AspNetCore.Mvc.Controllers;
using Microsoft.AspNetCore.Mvc.Filters;

namespace Bunny.Common.Filter;

/// <summary>
///     自定事务注解
/// </summary>
public class TransactionIAsyncActionFilter : IAsyncActionFilter
{
    private static readonly ILog Log = LogManager.GetLogger(typeof(TransactionIAsyncActionFilter));

    public async Task OnActionExecutionAsync(ActionExecutingContext context, ActionExecutionDelegate next)
    {
        var actionDesc = context.ActionDescriptor as ControllerActionDescriptor;
        // 如果加上 【TransactionAttribute】 注解才执行操作
        var hasNotTransactionalAttribute = actionDesc?.MethodInfo.IsDefined(typeof(TransactionAttribute), false) ?? false;

        // 如果没有加上自定义注解直接跳过
        if (hasNotTransactionalAttribute)
        {
            await next();
            return;
        }

        // 开启事务 
        using var txScope = new TransactionScope(TransactionScopeAsyncFlowOption.Enabled);
        var result = await next();

        // 没有出错，直接标记完成
        if (result.Exception == null)
        {
            txScope.Complete();
        }
        else
        {
            // 输出执行数据库操作产生事务原因
            var errorMessage = $"执行数据库出错 用户Id：[{BaseContext.GetUserId()}] token [{BaseContext.GetToken()}]:{result.Exception}";
            Log.Error(errorMessage);
        }
    }
}